/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved

	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.

	This file is under Perforce control
	$Id: //depot/express/fcs992/include/avs/mem_defs.h#1 $
*/


/*-------------------------------------------------------------------------
 * Dynamic memory allocation is always a problem area.  There are
 * many ways to make mistakes:
 *
 *	* Write beyond end of memory allocated.
 *	* Use memory after it has been freed.
 *	* Assume that 'malloc' clears memory.
 *	* Not checking for NULL returned from 'malloc'.
 *	* Specify single argument to 'calloc'.
 *	* Failure to free memory (a "memory leak").
 *
 * These macros insert a layer of error checking routines between the
 * application and the system memory allocation routines.
 * For instance if you call 'malloc', this substitutes a call to 
 * 'MEMmalloc' instead.  It collects statistics, does error checking,
 * and optionally prints out debugging information.
 *
 * To activate this substitution, you must define the preprocessor
 * symbol MEM_DEFS_ENABLE.
 *
 * These substitutions can cause some strange syntax errors under
 * certain conditions.  See AVS Developer's Guide for more details.
 *-------------------------------------------------------------------------*/



#ifndef _MEM_DEFS_defined

#define _MEM_DEFS_defined 1

#ifdef MEM_DEFS_ENABLE
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>

/* on IBM, these two funcs are defined for inline code - this gets rid of
   warning: */
#ifdef strcmp
#undef strcmp
#endif
#ifdef strcpy
#undef strcpy
#endif
#ifdef strcat
#undef strcat
#endif
#ifdef strlen
#undef strlen
#endif

#ifdef sparc
/* to avoid include of /usr/include/malloc.h which clashes with mem_defs.h */
#define __malloc_h
#endif

#ifndef avsargs_h
#include <avs/avsargs.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

AVS_EXTERN( char *MEMmalloc, (unsigned,char*,int));
AVS_EXTERN( void MEMfree, (void*,char*,int));
AVS_EXTERN( char *MEMrealloc, (void*,unsigned,char*,int));
AVS_EXTERN( char *MEMcalloc, (unsigned,unsigned,char*,int));
AVS_EXTERN( char *MEMstrdup, (char*,char*,int));

AVS_EXTERN( int MEMstrcmp, (char*,char*,char*,int));
AVS_EXTERN( int MEMstrncmp, (char*,char*,int,char*,int));
AVS_EXTERN( int MEMstrlen, (char*,char*,int));
AVS_EXTERN( char *MEMstrcpy, (char*,char*,char*,int));
AVS_EXTERN( char *MEMstrcat, (char*,char*,char*,int));
AVS_EXTERN( char *MEMstrchr, (char*,int,char*,int));
AVS_EXTERN( char *MEMstrrchr, (char*,int,char*,int));

#ifdef __cplusplus
}
#endif

#define malloc(size) MEMmalloc((size),__FILE__,__LINE__)
#if stellar	/* Compiler can't deal with "void*". */
#define free(ptr) { MEMfree((ptr),__FILE__,__LINE__); (ptr) = NULL; }
#else
#define free(ptr) { MEMfree((ptr),__FILE__,__LINE__); (ptr) = (void*)0x55555555; }
#endif
#define realloc(ptr,size) MEMrealloc((ptr),(size),__FILE__,__LINE__)
#define calloc(nelem,elsize) MEMcalloc((nelem),(elsize),__FILE__,__LINE__)
#define strdup(ptr) MEMstrdup((ptr),__FILE__,__LINE__)

#define strcmp(s1,s2) MEMstrcmp((s1),(s2),__FILE__,__LINE__)
#define strncmp(s1,s2,n) MEMstrncmp((s1),(s2),(n),__FILE__,__LINE__)
#define strlen(s) MEMstrlen((s),__FILE__,__LINE__)
#define strcpy(s1,s2) MEMstrcpy((s1),(s2),__FILE__,__LINE__)
#define strcat(s1,s2) MEMstrcat((s1),(s2),__FILE__,__LINE__)
/*#define strchr(s,c) MEMstrchr((s),(c),__FILE__,__LINE__)*/
/*#define strrchr(s,c) MEMstrrchr((s),(c),__FILE__,__LINE__)*/

#else

/*
 * Forgeting to declare pointer-returning functions such as 
 * malloc or strdup can cause problems on some systems.
 */

#include <malloc.h>
#include <string.h>

#endif	/* ifdef MEM_DEFS_ENABLE */

#endif 	/* ifndef _MEM_DEFS_defined */

